// passer de double a double ne change pas le resultat
// Practical Astronomy with your Calculator
// 3rd Ed, Peter Duffet-Smith, Cambridge Univ Press (1988)

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define DEBUG {}
//#define printf {}

#define epsilon_g (279.403303) // degs
#define omega_g (282.768422)   // degs
#define ecc (0.016713)
#define epsilon (23.43929)     // epoch 2000

#define mylat (+47.20)    // Besancon, France
#define mylon (+06.00)
//#define mylat (+42.37)  // Boston, MA
//#define mylon (-71.05)

// ATTENTION : int = 16 bits sur MSP430
double jours(short an,short mois,short jours) 
{                                     // mois=0 si on a deja calcule les jours
 short jmois[12]={0,31,59,90,120,151,181,212,243,273,304,334};
 short i=0,cpt=0; 
 for (i=1990;i<an;i++) {cpt+=365;if ((i%4==0)&&(i!=2000)) cpt+=1;}
 if (mois!=0) {
    cpt+=jmois[mois-1];
    if ((i%4==0)&&(i!=2000)&&(mois>1)) cpt++;
    }
 cpt+=jours;
 return((double)(cpt));
 }

void borne24(double *machin)
{while (*machin>24.) *machin-=24.;
 while (*machin<0.)  *machin+=24.;
}

void borne(double *machin)
{while (*machin>360.) *machin-=360.;
 while (*machin<0.)   *machin+=360.;
}

void ecliptic_to_equatorial(double* lambda,double *delta)
{double sin_delta,x,y;
 *lambda*=(M_PI/180.);
 sin_delta=sin(epsilon*M_PI/180.)*sin(*lambda);
 *delta=asin(sin_delta)*180./M_PI; 
 y=sin(*lambda)*cos(epsilon*M_PI/180.);
 x=cos(*lambda);
 *lambda=atan(y/x)*180./M_PI;  // alpha deg
 if (x<0.) *lambda+=180; borne(lambda);  // chap. 27 p.41
 DEBUG("x=%f y=%f => alpha=%f\n",x,y,*lambda);
}

unsigned long sunriseset(unsigned short year,unsigned short month,unsigned short day)  
// fournit UTCsunset et UTSsunrise pour D=date (jours depuis 010190)
{double S,T,T0,D,N,Msol,Ec,lsol,lsolp,beta,betap,GSTr1,GSTs1,GSTr2,GSTs2,H,GSTs,GSTr,T0p;
 unsigned long res;
 D=jours(year,month,day); // 17 aout 2007 = 6437
 DEBUG("jours=%5.0f since 01/01/1990\n",D);
 //D=-1461.+69.;// exemple p.95: alpha=23.332801 delta=-4.309187  10 March 1986
 //D=-522.;     // exemple p.91:       8h26'4"         19deg12'42"
 //D=-3444.;    // exemple p.88:       8h25'46"        19deg13'48"
 N=360./365.242191*D;
 Msol=N+epsilon_g-omega_g;borne(&Msol);
 Ec=360./M_PI*ecc*sin(Msol/180.*M_PI);
 lsol=N+Ec+epsilon_g;     borne(&lsol);
 lsolp=lsol+0.985647;
 DEBUG("lsol=%f\n",lsol);
 ecliptic_to_equatorial(&lsol,&beta);
 lsol/=15.;
 DEBUG("alpha=%f delta=%f\n",lsol,beta);
 DEBUG("lsolp=%f \n",lsolp);
 ecliptic_to_equatorial(&lsolp,&betap);
 lsolp/=15.;
 DEBUG("alphap=%f deltap=%f\n",lsolp,betap);
 // chap 33, p.52
 H=acos(-tan(beta *M_PI/180.)*tan(mylat*M_PI/180.))*180./M_PI/15.;
 GSTr1=24.+lsol -H-mylon/15.;borne24(&GSTr1);
 GSTs1=lsol+H-mylon/15.;     borne24(&GSTs1);
 H=acos(-tan(betap*M_PI/180.)*tan(mylat*M_PI/180.))*180./M_PI/15.;
 GSTr2=24.+lsolp-H-mylon/15.;borne24(&GSTr2);
 GSTs2=lsolp+H-mylon/15.;    borne24(&GSTs2);
 if (GSTr1>GSTr2) {GSTr2+=24.;GSTs2+=24.;}
// calculer T00 chap 12 p.17
  S=D-3653.5;
  T=S/36525.;
  T0=6.697374558+(2400.051336*T)+(0.000025862*T*T);borne24(&T0);
  T0p=T0-((mylon/15.)*1.002738);while (T0p<0.) T0p+=24.;
  if (GSTr1<T0p) {GSTr2+=24.;GSTr1+=24.;}
  if (GSTs1<T0p) {GSTs2+=24.;GSTs1+=24.;}
 DEBUG("GSTr1=%f GSTs1=%f\n",GSTr1,GSTs1);
 DEBUG("GSTr2=%f GSTs2=%f\n",GSTr2,GSTs2);
 DEBUG("T00=%lf T00p=%lf \n",T0,T0p);
 GSTs=(24.07*GSTs1-T0*(GSTs2-GSTs1))/(24.07+GSTs1-GSTs2);
 GSTr=(24.07*GSTr1-T0*(GSTr2-GSTr1))/(24.07+GSTr1-GSTr2);
 DEBUG("GSTr=%f GSTs=%f\n",(GSTr),(GSTs));
 GSTr-=T0;borne24(&GSTr);GSTr*=0.9972695663;
 GSTs-=T0;borne24(&GSTs);GSTs*=0.9972695663;
 GSTr-=0.0753; // LATITUDE DEPENDENT
 GSTs+=0.0753; // LATITUDE DEPENDENT
 // {hrise minrise hset minset}
 res=(unsigned long)(rintf(floorf(GSTr)))*256*256*256+(unsigned long)(rintf((GSTr-floor(GSTr))*60.))*256*256+(unsigned long)(floor(GSTs))*256+(unsigned long)rintf((GSTs-floor(GSTs))*60.);
 return(res);
}

/*
int main(int argc,char **argv)
{unsigned long res;
 unsigned short k,hrise,minrise,hset,minset;
 if (argc>1)
   res=sunriseset(2007,0,atoi(argv[1]));
 else
   for (k=0;k<256;k++) {
   res=sunriseset(2007,0,k);
 hrise=  (res&0xFF000000)/(256*256*256);
 minrise=(res&0x00FF0000)/(256*256);
 hset=   (res&0x0000FF00)/256;
 minset= (res&0x000000FF);
 printf("day=%d UTr=%dh%dmin, UTs=%dh%dmin\n",k,hrise,minrise,hset,minset);
 }
}
*/
